@@ -1,5 +1,5 @@
 /* BFD back-end for Intel 386 PE IMAGE COFF files.
-   Copyright (C) 1995-2017 Free Software Foundation, Inc.
+   Copyright (C) 1995-2024 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -21,12 +21,12 @@
 #include "sysdep.h"
 #include "bfd.h"
 
-#define TARGET_SYM 		i386_pei_vec
-#define TARGET_NAME 		"pei-i386"
+#define TARGET_SYM		i386_pei_vec
+#define TARGET_NAME		"pei-i386"
 #define COFF_IMAGE_WITH_PE
 #define COFF_WITH_PE
-#define PCRELOFFSET 		TRUE
-#define TARGET_UNDERSCORE 	'_'
+#define PCRELOFFSET		true
+#define TARGET_UNDERSCORE	'_'
 /* Long section names not allowed in executable images, only object files.  */
 #define COFF_LONG_SECTION_NAMES 0
 #define COFF_SUPPORT_GNU_LINKONCE
