@@ -1,9 +1,9 @@
-/* Copyright (C) 2000, 2003 Free Software Foundation
+/* Copyright (C) 2000-2024 Free Software Foundation, Inc.
    Contributed by Alexandre Oliva <aoliva@cygnus.com>
 
    This file is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful, but
@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
 /* Generator of tests for Maverick.
 
