@@ -1,5 +1,5 @@
 /* MIPS ELF support for BFD.
-   Copyright (C) 1993-2014 Free Software Foundation, Inc.
+   Copyright (C) 1993-2024 Free Software Foundation, Inc.
 
    By Ian Lance Taylor, Cygnus Support, <ian@cygnus.com>, from
    information in the System V Application Binary Interface, MIPS
@@ -30,6 +30,10 @@
 
 #include "elf/reloc-macros.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Relocation types.  */
 START_RELOC_NUMBERS (elf_mips_reloc_type)
   RELOC_NUMBER (R_MIPS_NONE, 0)
@@ -110,7 +114,8 @@ START_RELOC_NUMBERS (elf_mips_reloc_type)
   RELOC_NUMBER (R_MIPS16_TLS_GOTTPREL, 110)
   RELOC_NUMBER (R_MIPS16_TLS_TPREL_HI16, 111)
   RELOC_NUMBER (R_MIPS16_TLS_TPREL_LO16, 112)
-  FAKE_RELOC (R_MIPS16_max, 113)
+  RELOC_NUMBER (R_MIPS16_PC16_S1, 113)
+  FAKE_RELOC (R_MIPS16_max, 114)
   /* These relocations are specific to VxWorks.  */
   RELOC_NUMBER (R_MIPS_COPY, 126)
   RELOC_NUMBER (R_MIPS_JUMP_SLOT, 127)
@@ -218,52 +223,70 @@ END_RELOC_NUMBERS (R_MIPS_maxext)
 #define EF_MIPS_ARCH		0xf0000000
 
 /* -mips1 code.  */
-#define E_MIPS_ARCH_1		0x00000000
+#define EF_MIPS_ARCH_1		0x00000000
 
 /* -mips2 code.  */
-#define E_MIPS_ARCH_2		0x10000000
+#define EF_MIPS_ARCH_2		0x10000000
 
 /* -mips3 code.  */
-#define E_MIPS_ARCH_3		0x20000000
+#define EF_MIPS_ARCH_3		0x20000000
 
 /* -mips4 code.  */
-#define E_MIPS_ARCH_4		0x30000000
+#define EF_MIPS_ARCH_4		0x30000000
 
 /* -mips5 code.  */
-#define E_MIPS_ARCH_5           0x40000000
+#define EF_MIPS_ARCH_5           0x40000000
 
 /* -mips32 code.  */
-#define E_MIPS_ARCH_32          0x50000000
+#define EF_MIPS_ARCH_32          0x50000000
 
 /* -mips64 code.  */
-#define E_MIPS_ARCH_64          0x60000000
+#define EF_MIPS_ARCH_64          0x60000000
 
 /* -mips32r2 code.  */
-#define E_MIPS_ARCH_32R2        0x70000000
+#define EF_MIPS_ARCH_32R2        0x70000000
 
 /* -mips64r2 code.  */
-#define E_MIPS_ARCH_64R2        0x80000000
+#define EF_MIPS_ARCH_64R2        0x80000000
 
 /* -mips32r6 code.  */
-#define E_MIPS_ARCH_32R6        0x90000000
+#define EF_MIPS_ARCH_32R6        0x90000000
 
 /* -mips64r6 code.  */
-#define E_MIPS_ARCH_64R6        0xa0000000
+#define EF_MIPS_ARCH_64R6        0xa0000000
 
 /* The ABI of the file.  Also see EF_MIPS_ABI2 above. */
 #define EF_MIPS_ABI		0x0000F000
 
 /* The original o32 abi. */
-#define E_MIPS_ABI_O32          0x00001000
+#define EF_MIPS_ABI_O32          0x00001000
 
 /* O32 extended to work on 64 bit architectures */
-#define E_MIPS_ABI_O64          0x00002000
+#define EF_MIPS_ABI_O64          0x00002000
 
 /* EABI in 32 bit mode */
-#define E_MIPS_ABI_EABI32       0x00003000
+#define EF_MIPS_ABI_EABI32       0x00003000
 
 /* EABI in 64 bit mode */
-#define E_MIPS_ABI_EABI64       0x00004000
+#define EF_MIPS_ABI_EABI64       0x00004000
+
+/* In order to support backwards compatibility we also
+   define the old versions of some of these constants.  */
+#define E_MIPS_ARCH_1     EF_MIPS_ARCH_1
+#define E_MIPS_ARCH_2     EF_MIPS_ARCH_2
+#define E_MIPS_ARCH_3     EF_MIPS_ARCH_3
+#define E_MIPS_ARCH_4     EF_MIPS_ARCH_4
+#define E_MIPS_ARCH_5     EF_MIPS_ARCH_5
+#define E_MIPS_ARCH_32    EF_MIPS_ARCH_32
+#define E_MIPS_ARCH_64    EF_MIPS_ARCH_64
+#define E_MIPS_ARCH_32R2  EF_MIPS_ARCH_32R2
+#define E_MIPS_ARCH_64R2  EF_MIPS_ARCH_64R2
+#define E_MIPS_ARCH_32R6  EF_MIPS_ARCH_32R6
+#define E_MIPS_ARCH_64R6  EF_MIPS_ARCH_64R6
+#define E_MIPS_ABI_O32    EF_MIPS_ABI_O32
+#define E_MIPS_ABI_O64    EF_MIPS_ABI_O64
+#define E_MIPS_ABI_EABI32 EF_MIPS_ABI_EABI32
+#define E_MIPS_ABI_EABI64 EF_MIPS_ABI_EABI64
 
 
 /* Machine variant if we know it.  This field was invented at Cygnus,
@@ -276,25 +299,54 @@ END_RELOC_NUMBERS (R_MIPS_maxext)
    00 - 7F should be left for a future standard;
    the rest are open. */
 
-#define E_MIPS_MACH_3900	0x00810000
-#define E_MIPS_MACH_4010	0x00820000
-#define E_MIPS_MACH_4100	0x00830000
-#define E_MIPS_MACH_4650	0x00850000
-#define E_MIPS_MACH_4120	0x00870000
-#define E_MIPS_MACH_4111	0x00880000
-#define E_MIPS_MACH_SB1         0x008a0000
-#define E_MIPS_MACH_OCTEON	0x008b0000
-#define E_MIPS_MACH_XLR     	0x008c0000
-#define E_MIPS_MACH_OCTEON2	0x008d0000
-#define E_MIPS_MACH_OCTEON3	0x008e0000
-#define E_MIPS_MACH_5400	0x00910000
-#define E_MIPS_MACH_5900	0x00920000
-#define E_MIPS_MACH_5500	0x00980000
-#define E_MIPS_MACH_9000	0x00990000
-#define E_MIPS_MACH_LS2E        0x00A00000
-#define E_MIPS_MACH_LS2F        0x00A10000
-#define E_MIPS_MACH_LS3A        0x00A20000
-
+#define EF_MIPS_MACH_3900	0x00810000
+#define EF_MIPS_MACH_4010	0x00820000
+#define EF_MIPS_MACH_4100	0x00830000
+#define EF_MIPS_MACH_ALLEGREX	0x00840000
+#define EF_MIPS_MACH_4650	0x00850000
+#define EF_MIPS_MACH_4120	0x00870000
+#define EF_MIPS_MACH_4111	0x00880000
+#define EF_MIPS_MACH_SB1         0x008a0000
+#define EF_MIPS_MACH_OCTEON	0x008b0000
+#define EF_MIPS_MACH_XLR     	0x008c0000
+#define EF_MIPS_MACH_OCTEON2	0x008d0000
+#define EF_MIPS_MACH_OCTEON3	0x008e0000
+#define EF_MIPS_MACH_5400	0x00910000
+#define EF_MIPS_MACH_5900	0x00920000
+#define EF_MIPS_MACH_IAMR2	0x00930000
+#define EF_MIPS_MACH_5500	0x00980000
+#define EF_MIPS_MACH_9000	0x00990000
+#define EF_MIPS_MACH_LS2E        0x00A00000
+#define EF_MIPS_MACH_LS2F        0x00A10000
+#define EF_MIPS_MACH_GS464       0x00A20000
+#define EF_MIPS_MACH_GS464E	0x00A30000
+#define EF_MIPS_MACH_GS264E	0x00A40000
+
+/* In order to support backwards compatibility we also
+   define the old versions of some of these constants.  */
+#define E_MIPS_MACH_3900      EF_MIPS_MACH_3900 
+#define E_MIPS_MACH_4010      EF_MIPS_MACH_4010 
+#define E_MIPS_MACH_4100      EF_MIPS_MACH_4100 
+#define E_MIPS_MACH_ALLEGREX  EF_MIPS_MACH_ALLEGREX 
+#define E_MIPS_MACH_4650      EF_MIPS_MACH_4650 
+#define E_MIPS_MACH_4120      EF_MIPS_MACH_4120 
+#define E_MIPS_MACH_4111      EF_MIPS_MACH_4111 
+#define E_MIPS_MACH_SB1       EF_MIPS_MACH_SB1 
+#define E_MIPS_MACH_OCTEON    EF_MIPS_MACH_OCTEON 
+#define E_MIPS_MACH_XLR       EF_MIPS_MACH_XLR 
+#define E_MIPS_MACH_OCTEON2   EF_MIPS_MACH_OCTEON2 
+#define E_MIPS_MACH_OCTEON3   EF_MIPS_MACH_OCTEON3 
+#define E_MIPS_MACH_5400      EF_MIPS_MACH_5400 
+#define E_MIPS_MACH_5900      EF_MIPS_MACH_5900 
+#define E_MIPS_MACH_IAMR2     EF_MIPS_MACH_IAMR2 
+#define E_MIPS_MACH_5500      EF_MIPS_MACH_5500 
+#define E_MIPS_MACH_9000      EF_MIPS_MACH_9000 
+#define E_MIPS_MACH_LS2E      EF_MIPS_MACH_LS2E 
+#define E_MIPS_MACH_LS2F      EF_MIPS_MACH_LS2F 
+#define E_MIPS_MACH_GS464     EF_MIPS_MACH_GS464 
+#define E_MIPS_MACH_GS464E    EF_MIPS_MACH_GS464E 
+#define E_MIPS_MACH_GS264E    EF_MIPS_MACH_GS264E 
+
 /* Processor specific section indices.  These sections do not actually
    exist.  Symbols with a st_shndx field corresponding to one of these
    values have a special meaning.  */
@@ -444,6 +496,9 @@ END_RELOC_NUMBERS (R_MIPS_maxext)
 /* ABI related flags section.  */
 #define SHT_MIPS_ABIFLAGS	0x7000002a
 
+/* GNU style symbol hash table with xlat.  */
+#define SHT_MIPS_XHASH		0x7000002b
+
 /* A section of type SHT_MIPS_LIBLIST contains an array of the
    following structure.  The sh_link field is the section index of the
    string table.  The sh_info field is the number of entries in the
@@ -549,11 +604,11 @@ typedef union
 typedef struct
 {
   /* Mask of general purpose registers used.  */
-  unsigned long ri_gprmask;
+  uint32_t ri_gprmask;
   /* Mask of co-processor registers used.  */
-  unsigned long ri_cprmask[4];
+  uint32_t ri_cprmask[4];
   /* GP register value for this object file.  */
-  long ri_gp_value;
+  uint32_t ri_gp_value;
 } Elf32_RegInfo;
 
 /* The external version of the Elf_RegInfo structure.  */
@@ -748,6 +803,12 @@ extern void bfd_mips_elf32_swap_reginfo_out
 
 /* Points to the base of a writable PLT.  */
 #define DT_MIPS_RWPLT          0x70000034
+
+/* Relative offset of run time loader map, used for debugging.  */
+#define DT_MIPS_RLD_MAP_REL    0x70000035
+
+/* Address of .MIPS.xhash section.  */
+#define DT_MIPS_XHASH	       0x70000036
 
 /* Flags which may appear in a DT_MIPS_FLAGS entry.  */
 
@@ -991,9 +1052,9 @@ typedef struct
   /* Size of option descriptor, including header.  */
   unsigned char size;
   /* Section index of affected section, or 0 for global option.  */
-  unsigned short section;
+  uint16_t section;
   /* Information specific to this kind of option.  */
-  unsigned long info;
+  uint32_t info;
 } Elf_Internal_Options;
 
 /* MIPS ELF option header swapping routines.  */
@@ -1057,13 +1118,13 @@ typedef struct
 typedef struct
 {
   /* Mask of general purpose registers used.  */
-  unsigned long ri_gprmask;
+  uint32_t ri_gprmask;
   /* Padding.  */
-  unsigned long ri_pad;
+  uint32_t ri_pad;
   /* Mask of co-processor registers used.  */
-  unsigned long ri_cprmask[4];
+  uint32_t ri_cprmask[4];
   /* GP register value for this object file.  */
-  bfd_vma ri_gp_value;
+  uint64_t ri_gp_value;
 } Elf64_Internal_RegInfo;
 
 /* ABI Flags structure version 0.  */
@@ -1093,7 +1154,7 @@ typedef struct
   unsigned char flags2[4];
 } Elf_External_ABIFlags_v0;
 
-typedef struct
+typedef struct elf_internal_abiflags_v0
 {
   /* Version of flags structure.  */
   unsigned short version;
@@ -1224,14 +1285,22 @@ extern void bfd_mips_elf_swap_abiflags_v0_out
 #define AFL_ASE_MIPS16       0x00000400 /* MIPS16 ASE.  */
 #define AFL_ASE_MICROMIPS    0x00000800 /* MICROMIPS ASE.  */
 #define AFL_ASE_XPA          0x00001000 /* XPA ASE.  */
-#define AFL_ASE_MASK         0x00001fff /* All ASEs.  */
+#define AFL_ASE_DSPR3        0x00002000 /* DSP R3 ASE.  */
+#define AFL_ASE_MIPS16E2     0x00004000 /* MIPS16e2 ASE.  */
+#define AFL_ASE_CRC          0x00008000 /* CRC ASE.  */
+#define AFL_ASE_RESERVED1    0x00010000 /* Reserved by MIPS Tech for WIP.  */
+#define AFL_ASE_GINV         0x00020000 /* GINV ASE.  */
+#define AFL_ASE_LOONGSON_MMI 0x00040000 /* Loongson MMI ASE.  */
+#define AFL_ASE_LOONGSON_CAM 0x00080000 /* Loongson CAM ASE.  */
+#define AFL_ASE_LOONGSON_EXT 0x00100000 /* Loongson EXT instructions.  */
+#define AFL_ASE_LOONGSON_EXT2 0x00200000 /* Loongson EXT2 instructions.  */
+#define AFL_ASE_MASK         0x003effff /* All ASEs.  */
 
 /* Values for the isa_ext word of an ABI flags structure.  */
 
 #define AFL_EXT_XLR           1  /* RMI Xlr instruction.  */
 #define AFL_EXT_OCTEON2       2  /* Cavium Networks Octeon2.  */
 #define AFL_EXT_OCTEONP       3  /* Cavium Networks OcteonP.  */
-#define AFL_EXT_LOONGSON_3A   4  /* Loongson 3A.  */
 #define AFL_EXT_OCTEON        5  /* Cavium Networks Octeon.  */
 #define AFL_EXT_5900          6  /* MIPS R5900 instruction.  */
 #define AFL_EXT_4650          7  /* MIPS R4650 instruction.  */
@@ -1246,6 +1315,8 @@ extern void bfd_mips_elf_swap_abiflags_v0_out
 #define AFL_EXT_5500         16  /* NEC VR5500 instruction.  */
 #define AFL_EXT_LOONGSON_2E  17  /* ST Microelectronics Loongson 2E.  */
 #define AFL_EXT_LOONGSON_2F  18  /* ST Microelectronics Loongson 2F.  */
+#define AFL_EXT_OCTEON3      19  /* Cavium Networks Octeon3.  */
+#define AFL_EXT_INTERAPTIV_MR2 20  /* Imagination interAptiv MR2.  */
 
 /* Masks for the flags1 word of an ABI flags structure.  */
 #define AFL_FLAGS1_ODDSPREG   1	 /* Uses odd single-precision registers.  */
@@ -1294,6 +1365,10 @@ enum
   /* Using -mips32r2 -mfp64 -mno-odd-spreg.  */
   Val_GNU_MIPS_ABI_FP_64A = 7,
 
+  /* This is reserved for backward-compatibility with an earlier
+     implementation of the MIPS NaN2008 functionality.  */
+  Val_GNU_MIPS_ABI_FP_NAN2008 = 8,
+
   /* Values defined for Tag_GNU_MIPS_ABI_MSA.  */
 
   /* Not tagged or not using any ABIs affected by the differences.  */
@@ -1303,4 +1378,8 @@ enum
   Val_GNU_MIPS_ABI_MSA_128 = 1,
 };
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _ELF_MIPS_H */
